@@ -5,7 +5,6 @@
 
 import logging
 from numbers import Number
-from collections.abc import Iterable
 
 import pandas as pd
 import numpy as np
@@ -22,21 +21,7 @@ from   scipy.stats     import linregress
 from .. import standard_functions as std
 
 
-# Helper functions
-
-def convert_beta_temperature( p ):
-    """
-    Converts beta (coldness) to temperature and vice versa.
-
-    :param p: Temperature in Kelvin or beta (coldness).
-    :returns: Beta (coldness) if temperature was given,
-
-        Temperature in Kelvin if beta was given.
-    """
-    return 1/( phys.physical_constants[ 'Boltzmann constant in eV/K' ][ 0 ]* p )
-
-
-# ## Distributions
+# Distributions
 
 
 def gaussian_distribution( mu = 0, sigma = 1, x = None ):
@@ -224,27 +209,33 @@ def peak_position( df, start = None, end = None ):
     :param df: A Pandas DataFrame containing the spectrum.
     :param start: Lower bound of the search range, or None. [Default: None]
     :param end: Upper bound of the search range, or None. [Default: None]
-    :returns: A Pandas DataSeries container positions of the max.
+    :returns: A Pandas Series container positions of the max.
     """
     peak = df.loc[ start : end ].idxmax()
     if not isinstance( peak, pd.Series ):
         # single columns passed in, transform back to Series
-        peak = pd.Series( peak, index = df.index, name = 'peak' ).rename( 'peak' )
+        peak = pd.Series( peak, index = df.index, name = 'peak' )
 
-    return peak
+    return peak.rename( 'peak' )
 
 
-def center_of_mass( df ):
+def center_of_mass( df, abs = True ):
     """
     Retuns a Pandas Series of the center of mass for the spectra.
 
     :param df: DataFrame to calaculate the center of mass on.
+    :param abs: Take center of mass using absolute value of values.
+        [Default: True]
     :returns: Pandas Series of center of masses.
     """
     idx = df.index
-    weights = df/ df.sum()
-    com = weights.multiply( idx, axis = 0 ).sum()
-    return com
+    total_weight = std.integrate_df( df )
+    weights = df/ total_weight
+    
+    com = weights.multiply( idx, axis = 0 )
+    com = std.integrate_df( com )
+
+    return com.rename( 'com' )
 
 
 def integrated_intensity( df, start = None, end = None ):
@@ -257,8 +248,8 @@ def integrated_intensity( df, start = None, end = None ):
     :returns: A Pandas DataFrame of spectral areas.
     """
     df = df.loc[ start : end ]
-    return df.apply( lambda datum: integrate.simps( datum ) ).rename( 'area' )
-
+    df = std.integrate_df( df )
+    return df.rename( 'area' )
 
 def peak_analysis( df, groups = None, start = None, end = None ):
     """
@@ -281,260 +272,30 @@ def peak_analysis( df, groups = None, start = None, end = None ):
 
     if groups is None:
         # return data sample by sample, no statistics
-        return pd.concat(
-
+        pdf = pd.concat(
             [ peaks, fw, area ],
-
             axis = 1,
-
             keys = [ 'peak', 'fwhm', 'area' ]
-
         )
 
-    # group analysis, include statistics
-    peaks = peaks.groupby( groups )
-    fw    = fwhm.groupby(  groups )
-    area  = area.groupby(  groups )
 
-    return pd.concat( [
-
-        peaks.mean().rename( ( 'peak', 'mean' ) ),
-        peaks.std().rename(  ( 'peak', 'std' ) ),
-        fw.mean().rename( ( 'fwhm', 'mean' ) ),
-        fw.std().rename(  ( 'fwhm', 'std' ) ),
-        area.mean().rename( ( 'area', 'mean' ) ),
-        area.std().rename(  ( 'area', 'std' ) )
-    ], axis = 1 )
-
-
-def extract_temperature(
-    df,
-    value_threshold = 0,
-    grad_threshold = 10,
-    curve_threshold = 1e5,
-    side = 'high',
-    mask_window = 75,
-):
-    """
-    Finds the temperature coefficient from a PL curve.
-    Performs a linear fit on the log of PL spectra on the low or high energy side.
-    The fit is performed on an area with gradient higher than the given threshold,
-    and curvature less that the given threshold.
-
-    :param df: DataFrame of PL specra indexed by energy.
-    :param value_threshold: Minimum value relative to max to consider.
-        [Default: 0]
-    :param grad_threshold: Minimum gradient threshold. [Default: 40]
-    :param curve_threshold: Maximum curvature threshold. [Default: 1000]
-    :param side: 'low' for low energy, 'high' for high energy. [Default: 'high']
-    :param mask_window: Smoothing window for data mask. [Default: 75]
-    :returns: Dictionary of tuples of ( temperature, linear fit ).
-        If no valid data for a particular dataset vlaue is None.
-
-    """
-    logger = logging.getLogger( __name__ )
-    df = df.copy()
-
-    # calculate needed data
-    ldf = df.apply( np.log ).replace( [ -np.inf, np.inf ], np.nan ).dropna( how = 'all' )
-    gdf = ldf.apply( std.df_grad )
-    cdf = gdf.apply( std.df_grad )
-
-    fits = {}
-    for name, data in ldf.items():
-        mask = (
-            data.index < data.idxmax()
-            if side == 'low' else
-            data.index > data.idxmax()
-        )
-
-        if not np.any( mask ):
-            # no valid data
-            fits[ name ] = None
-            logger.info( f'No data for { name } due to side mask.' )
-            continue
-        
-        v_mask = ( 
-            ldf[ name ][ mask ].apply( np.exp ) > value_threshold
-            if value_threshold > 0 else
-            ldf[ name ][ mask ] > -np.inf
-        )
-
-        g_mask = (
-            gdf[ name ][ mask ] > grad_threshold
-            if side == 'low' else
-            gdf[ name ][ mask ] < -grad_threshold
-        )
-
-        c_mask = (
-            ( 0 > cdf[ name ][ mask ] ) &
-            ( cdf[ name ][ mask ] > -curve_threshold )
-        )
-
-        v_mask = std.smooth_mask( v_mask, window = mask_window )
-        g_mask = std.smooth_mask( g_mask, window = mask_window )
-        c_mask = std.smooth_mask( c_mask, window = mask_window )
-
-        tdf = data[ mask ]
-        tdf = tdf[ v_mask & g_mask & c_mask ]
-        tdf = tdf.dropna()
-        
-        if tdf.shape[ 0 ] == 0:
-            # no data
-            fits[ name ] = None
-            logger.info( f'No data for { name } due to masking.' )
-            continue
-
-        # valid data, fit
-        fit = linregress( x = tdf.index.values, y = tdf.values )
-        if np.isnan( fit.slope ):
-            # could not fit
-            fits[ name ] = None
-            logger.info( f'Could not fit { name }.' )
-            continue
-
-        beta = fit.slope
-        if side == 'high':
-            beta *= -1
-
-        temp = convert_beta_temperature( beta )
-        fits[ name ] = ( temp, fit )
-
-    return fits
-
-
-def differential_temperature( df, window = 11, normalize = True ):
-    """
-    Extracts the differential temperature from a DataFrame.
-
-    :param df: Pandas DataFrame of PL spectra.
-    :param window: Window size for linear fitting.
-        [Default: 11]
-    :returns: Pandas DataFrame of differential temperatures.
-    :raises ValueError: If window is smaller than 1.
-    :raises ValueError: If window is not odd valued.
-    """
-    def _temp_fit( row, data ):
-        ind_loc = data.index.get_loc( row.name )
-        tdf = data.iloc[ ind_loc - half_window : ind_loc + half_window + 1 ]
-        fit = linregress( tdf.index, tdf.values )
-        temp = -convert_beta_temperature( fit.slope )
-        return ( temp, fit )
-    
-    if window < 2:
-        raise ValueError( 'Window must be larger than 1.' )
-
-    if window% 2 == 0:
-        raise ValueError( 'Window must be odd.' )
-
-    half_window = int( ( window - 1 )/ 2 )
-    ldf = df.apply( np.log )
-    fdf = []
-    for name, data in ldf.items():
-        tdf = data.dropna().to_frame()
-        tdf = tdf.iloc[ half_window : -half_window ]
-        tdf = tdf.apply(
-            _temp_fit,
-            axis = 1,
-            args = ( data, ),
-            result_type = 'expand'
-        )
-        
-        tdf = tdf.rename( { 0: 'temperature', 1: 'fit' }, axis = 1 )
-        headers = [
-            ( *name, val ) if isinstance( name, Iterable ) else ( name, val )
-            for val in  tdf.columns.values
-        ]
-        
-        tdf.columns = pd.MultiIndex.from_tuples(
-            headers,
-            names = ( *df.columns.names, 'fits' )
-        )
-        
-        fdf.append( tdf )
-    
-    if len( fdf ) > 1:
-        fdf = pd.concat( fdf, axis = 1 )
-    
     else:
-        fdf = fdf[ 0 ]
+        # group analysis, include statistics
+        peaks = peaks.groupby( groups )
+        fw    = fwhm.groupby( groups )
+        area  = area.groupby( groups )
 
-    return fdf
-
-
-def differential_temperature_stats(
-    df,
-    grad_threshold = 10,
-    curve_threshold = 1e5,
-    side = 'high',
-    mask_window = 75,
-):
-    """
-    Returns statistics on differential temepratures.
-    Used in conjunction with #differential_temperature
-
-    :param df: DataFrame of differential temperatures indexed by energy.
-    :param grad_threshold: Maximum gradient threshold. [Default: 40]
-    :param curve_threshold: Maximum curvature threshold. [Default: 1000]
-    :param side: 'low' for low energy, 'high' for high energy. [Default: 'high']
-    :param mask_window: Smoothing window for data mask. [Default: 75]
-    :returns: Dictionary of tuples of ( temperature, linear fit ).
-        If no valid data for a particular dataset vlaue is None.
-
-    """
-    logger = logging.getLogger( __name__ )
-    df = df.copy()
-
-    # calculate needed data
-    gdf = df.apply( std.df_grad )
-    cdf = gdf.apply( std.df_grad )
-
-    stats = {}
-    for name, data in df.items():
-        g_data = gdf[ name ]
-        
-        g_mask = g_data.abs() < grad_threshold
-        g_mask = std.smooth_mask( g_mask, window = mask_window )
-
-        c_mask = cdf[ name ].abs() < curve_threshold 
-        c_mask = std.smooth_mask( c_mask, window = mask_window )
-
-        mask  = (
-            g_data.index < g_data.idxmin()
-            if side == 'low' else
-            g_data.index > g_data.idxmax()
-        )
-        
-        tdf = data[ mask & g_mask & c_mask ]
-        tdf = tdf.dropna()
-
-        if tdf.shape[ 0 ] == 0:
-            # no data
-            mean = None
-            stddev = None
-            floor = None
-
-        else:
-            mean = tdf.mean()
-            stddev = tdf.std()
-            floor = (
-                tdf.max()
-                if side == 'low' else
-                tdf.min()
-            )        
-
-        if not isinstance( name, Iterable ):
-            # normalize name to tuple if required
-            name = ( name, )
-
-        stats[ ( *name, 'mean' ) ] = mean
-        stats[ ( *name, 'std' ) ] = stddev
-        stats[ ( *name, 'min' ) ] = floor
-
-    stats = pd.Series( stats )
-    stats.index = stats.index.rename( ( *df.columns.names, 'metrics' ) )
-    return stats
+        pdf = pd.concat( [
+            peaks.mean().rename( ( 'peak', 'mean' ) ),
+            peaks.std().rename(  ( 'peak', 'std' ) ),
+            fw.mean().rename( ( 'fwhm', 'mean' ) ),
+            fw.std().rename(  ( 'fwhm', 'std' ) ),
+            area.mean().rename( ( 'area', 'mean' ) ),
+            area.std().rename(  ( 'area', 'std' ) )
+        ], axis = 1 )
 
+    pdf.index = pdf.index.rename( df.columns.names )
+    return pdf
 
 
 # ## Spectral functions
