@@ -851,9 +851,15 @@ def df_fit_function( fcn, param_names = None, guess = None, modify = None, **kwa
                 fit[ 0 ]
             ) )
 
+            variances = fit[ 1 ].diagonal()
+            stds = [
+                np.sqrt( v ) if ( v > 0 ) else np.nan
+                for v in variances
+            ]
+            
             stds = dict( zip(
                 [ ( param, 'std' ) for param in param_names ],
-                np.sqrt( fit[ 1 ].diagonal() )
+                stds
             ) )
 
             params.update( stds )
@@ -863,18 +869,19 @@ def df_fit_function( fcn, param_names = None, guess = None, modify = None, **kwa
         fits = pd.DataFrame( fits )
         fits.columns = fits.columns.rename( [ 'parameter', 'metric' ] )
 
-        if isinstance( df.index, pd.Index ):
-            fits.index = pd.Index(
-                fits.index.values,
-                name = mdf.index.name
-            )
-
-        elif isinstance( df.index, pd.MultiIndex ):
+        if isinstance( df.columns, pd.MultiIndex ):
+            # must go first as MultiIndex is also an Index
             fits.index = pd.MultiIndex.from_tuples(
                 fits.index.values,
                 names = mdf.columns.names
             )
 
+        elif isinstance( df.columns, pd.Index ):
+            fits.index = pd.Index(
+                fits.index.values,
+                name = mdf.index.name
+            )
+        
         else:
             raise TypeError( 'Unknown type of index encountered.')
 
@@ -967,18 +974,23 @@ def df_find_index_of_value( df, value, fit_window = 20, polydeg = 1 ):
     :param polydeg: Degree of the polynomial to use for fitting. [Default: 1] 
     :returns: A tuple of ( indices, fit )
         `indices` is Pandas Series of the index for each data series.
-        `fit` is the numpy.Polynomial that was used for the fit.
+        `fit` is the numpy.Polynomial that was used for the fit,
+            or None if the fit was not needed due to an exact match 
+            or did not converge.
     """
+    if isinstance( df, pd.Series ):
+        df = df.to_frame()
+
     df = df.sort_index()
 
-    index = pd.Series( index = df.columns, dtype = np.float64 )
+    val_index = pd.DataFrame( index = df.columns, columns = ( 'index', 'fit' ) )
     pos = df[ df > value ]
     neg = df[ df < value ]
     for name, data in df.items():
         # check if value exists in data
         val_ind = data.index[ data == value ]
         if val_ind.shape[ 0 ]:
-            index[ name ] = val_ind[ 0 ]
+            val_index.loc[ name ] = [ val_ind[ 0 ], None ]
             continue
 
         dpos = pos[ name ].dropna()
@@ -996,32 +1008,41 @@ def df_find_index_of_value( df, value, fit_window = 20, polydeg = 1 ):
 
         else:
             # one sided data
-            tdf = dneg.iloc[ -fit_window: ] if dneg.shape[ 0 ] else dpos.iloc[ :fit_window ]
+            tdf = (
+                dneg.iloc[ : fit_window ]
+                if dneg.shape[ 0 ] else
+                dpos.iloc[ -fit_window : ]
+            )
 
         if tdf.shape[ 0 ] < 3:
-            index[ name ] = np.nan
+            val_index.loc[ name ] = [ np.nan, None ]
             continue
 
         tdf = tdf.squeeze()
         try:
             fit = Polynomial.fit( tdf.index, tdf.values - value, deg = polydeg )  # shift values so value is a root
-        
+            
         except Exception as err:
-            print( err )
-            index[ name ] = np.nan
+            print( f'[{name}] {err}' )
+            val_index.loc[ name ] = [ np.nan, None ]
 
         else:
             roots = fit.roots()
+            if roots.shape[ 0 ] == 0:
+                # no roots found
+                val_index.loc[ name ] = [ np.nan, None ]
+                continue
 
             # find root nearest to index values
             root_dists = [
                 np.abs( tdf.index - root ).min()
                 for root in roots
             ]
-            root = roots[ np.argmin( root_dists) ]
-            index[ name ] = root
+            root = roots[ np.argmin( root_dists ) ]
+        
+            val_index.loc[ name ] = [ root, fit ]
 
-    return ( index.rename( df.index.names ), fit )
+    return val_index
 
 
 # Mask functions
@@ -1032,13 +1053,13 @@ def smooth_mask( mask, window = 10, fillna = 'backfill' ):
 
     :param mask: Mask to smooth.
     :param window: Smoothing window. [Default: 10]
-    :param fillna: Method to fill na values from smoothing,
-        or None to leave as na.
-        [See pandas.Series#fillna]
-        [Default: 'backfill']
+    :param fillna: True to fill NaN values from smoothing,
+        or False to leave as NaN.
+        [Default: True]
     :returns: Smoothed mask.
     """
     # convert False/True to 0/1
+    return_series = True
     if isinstance( mask, pd.DataFrame ):
         if mask.shape[ 1 ] > 1:
             raise TypeError( 'Mask can not have more than one column.' )
@@ -1050,15 +1071,23 @@ def smooth_mask( mask, window = 10, fillna = 'backfill' ):
 
     else:
         df = pd.Series( map( int, mask ) )
+        return_series = False
 
     # smooth values and convert back to False/True
-    df = df.rolling( window = window ).mean()
-    if fillna is not None:    
-        df = df.fillna( method = fillna )
-    df = df.apply( round )
-    df = df.apply( bool )
+    df = df.rolling( window = window, center = True ).mean()
+    if fillna:    
+        df = df.fillna( method = 'bfill' ).fillna( method = 'ffill' )
 
-    return df.values
+    df = df.apply( np.around )
+
+    nan_mask = df.isna().apply( np.logical_not )
+    df[ nan_mask ] = df[ nan_mask ].apply( bool )
+
+    if return_series:
+        return df
+
+    else:
+        return df.values
 
 
 def mask_from_threshold(
@@ -1517,9 +1546,43 @@ def integrate_df( df ):
         df = df.to_frame()
 
     x = df.index
-    data = {
-        name: trapezoid( vals, x )
-        for name, vals in df.items()
-    }
+    data = {}
+    for name, tdf in df.items():
+        tdf = tdf.dropna()
+        data[ name ] = trapezoid( tdf.values, tdf.index )
 
-    return pd.Series( data )
+    sd = pd.Series( data, name = 'area' )
+    sd.index = sd.index.rename( df.columns.names )
+
+    return sd
+
+
+def rescale_values( x, r_min, r_max, d_min = None, d_max = None ):
+    """
+    Rescale values to reference range 
+    using high and low key points.
+    
+    d_min <---- x0 ---- x1 ---- x2 ----> d_max
+                 \       \      |
+                  --      |     |  
+                    \     |     |
+          r_min <-- x0 -- x1 -- x2 --> r_max
+    
+    :param x: numpy.array of values to convert.
+    :param r_min: Range min value.
+    :param r_max: Range max value.
+    :param d_min: Domain min value. If None uses min value in `x`.
+        [Default: None]
+    :param d_max: Domain max value. If None uses max value in `x`.
+        [Default: None]
+    :returns: Rescaled values from domain to range.
+    """
+    if d_min is None:
+        d_min = x.min()
+        
+    if d_max is None:
+        d_max = x.max()
+        
+    scaled = ( x - d_min )/( d_max - d_min )
+    rng = r_max - r_min
+    return scaled* rng + r_min
